document.addEventListener('DOMContentLoaded', function() {
    // Retrieve from sessionStorage
    let projectId = sessionStorage.getItem('projectId');
    let projectName = sessionStorage.getItem('projectName');

    if (projectId && projectName) {
        console.log("Existing session data found:", projectName, projectId);
        fetchInternalAuditData(projectId, decodeURIComponent(projectName));
        populateAuditFindingsTable(projectId);
    } else {
        console.error('No project data found in sessionStorage on page reload.');
        // Handle case when no data is found - possibly redirect or prompt user
    }
});


function fetchInternalAuditData(projectId, projectName) {
    // Construct the URL with query parameters
    const url = `/fetch-ia-data?projectId=${encodeURIComponent(projectId)}`;

    // Send a GET request to the server
    fetch(url)
        .then(response => {
            if (!response.ok) {
                throw new Error('Network response was not ok');
            }
            return response.json();
        })
        .then(data => {
            console.log('Internal Audit Data:', data);
            updateProjectDetailsTable(data);
            updateHTMLWithData(data);
        })
        .catch(error => {
            console.error('Fetch error:', error);
            // Handle any errors that occurred during the fetch
        });
}

function updateProjectDetailsTable(data) {
    // Assuming 'data' contains the properties: project_name, date_of_submission, created_by, and customer_name

    // Select the table cells where the data needs to be displayed
    const projectNameCell = document.querySelector('#project-details tr:nth-child(1) td');
    const dateOfSubmissionCell = document.querySelector('#project-details tr:nth-child(2) td');
    const createdByCell = document.querySelector('#project-details tr:nth-child(3) td');
    const customerNameCell = document.querySelector('#project-details tr:nth-child(6) td');

    // Update the cells with the data
    if (projectNameCell) projectNameCell.textContent = data.project_name;
    if (dateOfSubmissionCell) dateOfSubmissionCell.textContent = data.date_of_submission;
    if (createdByCell) createdByCell.textContent = data.created_by;
    if (customerNameCell) customerNameCell.textContent = data.customer_name;
}

function updateHTMLWithData(data) {
    // Update Customer Name
    const customerNameElements = document.querySelectorAll('.customerNamePlaceholder');
    customerNameElements.forEach(element => {
        element.textContent = data.customer_name;
        element.style.fontWeight = 'bold';
    });

    // Update Date of Submission
    const dateOfSubmissionElements = document.querySelectorAll('.dateOfSubmissionPlaceholder');
    dateOfSubmissionElements.forEach(element => {
        element.textContent = data.date_of_submission;
        element.style.fontWeight = 'bold';
    });

    // Update Created By
    const createdByElements = document.querySelectorAll('.createdByPlaceholder');
    createdByElements.forEach(element => {
        element.textContent = data.created_by;
        element.style.fontWeight = 'bold';
    });

    // Update Month Started
    const monthStartedElements = document.querySelectorAll('.monthStartedPlaceholder');
    monthStartedElements.forEach(element => {
        element.textContent = data.month_started;
        element.style.fontWeight = 'bold';
    });

    // Update Month Finished
    const monthFinishedElements = document.querySelectorAll('.monthFinishedPlaceholder');
    monthFinishedElements.forEach(element => {
        element.textContent = data.month_finished;

        // Check if "data.month_finished" contains a year (e.g., "January 2024")
        const yearRegex = /\d{4}/;
        const yearMatch = data.month_finished.match(yearRegex);

        if (yearMatch) {
            // Extract the year and add 1 to it
            const currentYear = parseInt(yearMatch[0]);
            const nextYear = currentYear + 1;

            // Update the NextYearPlaceholder
            const nextYearElements = document.querySelectorAll('.NextYearPlaceholder');
            nextYearElements.forEach(nextYearElement => {
                nextYearElement.textContent = ' ' + data.month_finished.replace(yearRegex, nextYear);
                element.style.fontWeight = 'bold';
            });
        } else {
            // If no year is found, add 1 to the current year and update both placeholders
            const currentYear = new Date().getFullYear();
            const nextYear = currentYear + 1;

            const nextYearElements = document.querySelectorAll('.NextYearPlaceholder');
            nextYearElements.forEach(nextYearElement => {
                nextYearElement.textContent = ' ' + data.month_finished + ' ' + nextYear;
                element.style.fontWeight = 'bold';
            });
        }
    });
}

function populateAuditFindingsTable(projectId) {
    const nonConformities = [];
    const opportunitiesForImprovement = [];

    fetch(`/get-audit-findings?project_id=${projectId}`)
        .then(response => response.json())
        .then(data => {
            let ncCounter = 1; // Counter for Non-Conformities
            let ofiCounter = 1; // Counter for Opportunities for Improvement

            async function buildTopic(finding) {
                try {
                    const response = await fetch(`/get-control-text/${finding.control_id}`);
                    const controlTextData = await response.json();
                    const controlText = controlTextData.control_text;

                    if (finding.non_conformity) {
                        // Updated format for topic
                        const topic = `NC#${ncCounter} - ${finding.control_id} - ${controlText}`;
                        const findingText = `It has been found that ${finding.notes}`;
                        nonConformities.push({
                            control_id: finding.control_id,
                            notes: finding.notes,
                            finding: findingText,
                            topic: topic
                        });
                        ncCounter++;
                    }

                    if (finding.opportunity_for_improvement) {
                        // Updated format for topic
                        const topic = `OFI#${ofiCounter} - ${finding.control_id} - ${controlText}`;
                        const findingText = `It has been found that ${finding.notes}`;
                        opportunitiesForImprovement.push({
                            control_id: finding.control_id,
                            notes: finding.notes,
                            finding: findingText,
                            topic: topic
                        });
                        ofiCounter++;
                    }
                } catch (error) {
                    console.error('Error fetching control text:', error);
                }
            }

            const buildTopicPromises = data.map(finding => buildTopic(finding));

            Promise.all(buildTopicPromises)
                .then(() => {
                    function appendRow(tableBody, rowData) {
                        const row = tableBody.insertRow();
                        const topicCell = row.insertCell(0);
                        const findingCell = row.insertCell(1);

                        topicCell.textContent = rowData.topic;
                        findingCell.textContent = rowData.finding;
                    }

                    const tableBody = document.querySelector('#audit-findings tbody');
                    nonConformities.forEach(nc => {
                        appendRow(tableBody, nc);
                    });
                    opportunitiesForImprovement.forEach(ofi => {
                        appendRow(tableBody, ofi);
                    });
                })
                .catch(error => {
                    console.error('Error building topics:', error);
                });
        })
        .catch(error => {
            console.error('Error fetching audit findings:', error);
        });
}

function appendRow(tableBody, rowData) {
    console.log ("running");
    const row = tableBody.insertRow();
    const topicCell = row.insertCell(0);
    const findingCell = row.insertCell(1);
    const remediationCell = row.insertCell(2);

    topicCell.textContent = rowData.topic;
    findingCell.textContent = rowData.finding;

    // Create and append the select dropdown for the Remediation Plan
    const select = document.createElement('select');
    select.classList.add('form-control');  // Add styling class if you are using Bootstrap or similar CSS framework

    // Define dropdown options
    const options = ["No Action Needed", "Plan to Correct", "Under Review", "Corrected", "Custom Plan..."];
    options.forEach(option => {
        let optionElement = document.createElement('option');
        optionElement.value = option;
        optionElement.textContent = option;
        select.appendChild(optionElement);
    });

    // Add an event listener for handling custom plan input
    select.addEventListener('change', function() {
        if (this.value === "Custom Plan...") {
            const customPlan = prompt("Enter your custom remediation plan:");
            if (customPlan) {  // Only add if the user entered something
                const newOption = document.createElement('option');
                newOption.textContent = newOption.value = customPlan;
                this.appendChild(newOption);
                this.value = customPlan;  // Set the newly added option as the selected option
            } else {
                this.value = "No Action Needed";  // Default back if no input is provided
            }
        }
    });

    // Append the select element to the newly created cell for Remediation Plan
    remediationCell.appendChild(select);
}
