let complianceStatusData = {};
var projectId , complianceType;

document.addEventListener('DOMContentLoaded', function() {
    // First, fetch the session data needed
    fetch('/get-soa-session-data')
        .then(response => {
            if (!response.ok) {
                throw new Error('Failed to fetch session data');
            }
            return response.json();
        })
        .then(sessionData => {
            if (sessionData.error) {
                console.error('Error:', sessionData.error);
                alert('Session data could not be loaded.');
            } else {
                complianceType = sessionData.complianceType; // Use global variable directly
                projectId = sessionData.projectId;           // Use global variable directly
                console.log('Compliance Type:', complianceType);
                console.log('Project ID:', projectId);

                // Call fetchData with the retrieved session data
                fetchData(complianceType, projectId);
            }
        })
        .catch(error => {
            console.error('Error:', error);
            alert('Unable to fetch session data.');
        });

    // Save button event listener
    const saveButton = document.querySelector('.saveBtn');
    saveButton.addEventListener('click', function() {
        document.querySelectorAll('tr[data-control-id]').forEach(row => {
            submitSOAData(row);
        });
    });
});


function fetchData(complianceType, projectId) {
    Promise.all([
        fetch(`/check-soa-data/${complianceType}/${projectId}`)
            .then(response => {
                if (!response.ok && response.status === 404) {
                    // If specific data not found, load default data
                    return fetch('/soa-data-default').then(defaultResponse => defaultResponse.json());
                }
                return response.json();
            }),
        fetch(`/get-compliance-status/${projectId}`).then(response => response.json())
    ])
    .then(([soaData, complianceStatus]) => {
        console.log('SOA Data:', soaData);
        console.log('Compliance Status:', complianceStatus);
        updateApplicability(soaData, complianceStatus);
        populateTable(soaData);
    })
    .catch(error => {
        console.error('Error fetching data:', error);
        fetch('/soa-data-default')
            .then(response => response.json())
            .then(defaultData => populateTable(defaultData))
            .catch(finalError => console.error('Final fallback error:', finalError));
    });
}

function updateApplicability(soaData, complianceStatus) {
    const statusMap = complianceStatus.reduce((acc, item) => {
        acc[item.control_id] = item.control_status === 'Compliant' ? 'Yes' : 'No';
        return acc;
    }, {});

    soaData.forEach(item => {
        item.applicability = statusMap[item.control_id] || 'No'; // Default to 'No' if not specified
    });
}


function populateTable(data) {
    const tableBody = document.getElementById('complianceTableBody');
    tableBody.innerHTML = ''; // Clear existing rows

    data.sort((a, b) => {
        const partsA = a.control_id.split('.').map(Number);
        const partsB = b.control_id.split('.').map(Number);
        if (partsA[0] !== partsB[0]) return partsA[0] - partsB[0];
        return partsA[1] - partsB[1];
    });

    data.forEach(item => {
        const row = document.createElement('tr');
        row.setAttribute('data-control-id', item.control_id);

        const fields = [
            { key: 'control_id', editable: false },
            { key: 'control_text', editable: true },
            { key: 'applicability', editable: true },
            { key: 'remarks', editable: true },
            { key: 'current_control', editable: true },
            { key: 'accountability', editable: true }
        ];

        fields.forEach(({ key, editable }) => {
            const cell = document.createElement('td');
            if (editable) {
                const input = document.createElement('input');
                input.type = 'text';
                input.value = item[key] || '';
                input.className = 'editable';
                input.onchange = function() { item[key] = input.value; };
                cell.appendChild(input);
            } else {
                cell.textContent = item[key] || '';
            }
            row.appendChild(cell);
        });

        tableBody.appendChild(row);
    });
}

function prepareTableForExport(tableId) {
    const table = document.getElementById(tableId);
    const inputs = table.querySelectorAll('input');
    const originalInputs = [];

    inputs.forEach(input => {
        const textNode = document.createTextNode(input.value);
        originalInputs.push({ parent: input.parentNode, value: input.value });
        input.parentNode.replaceChild(textNode, input);
    });

    return originalInputs; // Return details for reverting after export
}

function exportTableToXLSX(tableId, filename) {
    // Convert inputs to text and keep the original inputs for reverting
    const originalInputs = prepareTableForExport(tableId);

    // Proceed with the export
    var table = document.getElementById(tableId);
    var wb = XLSX.utils.table_to_book(table, { sheet: "Sheet 1", raw: false });
    XLSX.writeFile(wb, filename, {
        complete: function() {
            // Revert the table after export
            revertTableAfterExport(tableId, originalInputs);
        }
    });
}

function revertTableAfterExport(tableId, originalInputs) {
    originalInputs.forEach(inputDetails => {
        const input = document.createElement('input');
        input.type = 'text';
        input.value = inputDetails.value;
        inputDetails.parent.textContent = ''; // Clear the text node
        inputDetails.parent.appendChild(input); // Reinsert the input
    });
}

function submitSOAData(row) {
    const controlId = row.getAttribute('data-control-id');
    const inputs = row.querySelectorAll('input');

    if (inputs.length < 5) {
        console.error('Not enough input fields:', inputs.length);
        return;
    }

    let soaData = {
        project_id: projectId,
        compliance_type: complianceType,
        control_id: controlId,
        control_text: inputs[0]?.value.trim() || '',
        applicability: inputs[1]?.value.trim() || '',
        remarks: inputs[2]?.value.trim() || '',
        current_control: inputs[3]?.value.trim() || '',
        accountability: inputs[4]?.value.trim() || ''
    };

    fetch('/submit-soa', {
        method: 'POST',
        headers: { 'Content-Type': 'application/json' },
        body: JSON.stringify(soaData)
    })
    .then(response => {
        if (!response.ok) throw new Error('Server responded with an error!');
        return response.json();
    })
    .then(data => {
        console.log('Data Submitted:', data);
    })
    .catch(error => console.error('Error submitting SOA data:', error));
}
