document.addEventListener('DOMContentLoaded', function() {
    // Assuming you have a function in your Flask app to get the user ID
    fetchUserIdFromSession();
});

var user_id;

function fetchUserIdFromSession() {
    // Make a GET request to a server endpoint that returns the session data
    fetch('/get-user-id', {
        method: 'GET',
        credentials: 'include',  // Include credentials (cookies) in the request
    })
    .then(response => response.json())
    .then(data => {
        if (data.user_id !== undefined) {
            user_id = data.user_id;
            fetchTrainingPlan(user_id);
        } else {
            console.error('User is not authenticated or session data is missing.');
        }
    })
    .catch(error => console.error('Error fetching user session data:', error));
}

function fetchTrainingPlan(user_Id) {
    fetch(`/api/user-training-plan/${user_Id}`)
        .then(response => {
            if (!response.ok) {
                throw new Error('Network response was not ok');
            }
            return response.json();
        })
        .then(data => {
            updateTrainingPlanUI(data); // Update training plan UI
            const barChartData = processDataForBarChart(data);
            createBarChart(barChartData); // Create the bar chart

            // Assuming you have a separate function or way to get test breakdown data
            // For example, if it's part of the same 'data' response:
            if (data.testBreakdownData) {
                updateTestBreakdownUI(data.testBreakdownData); // Update test breakdown UI
            }
        })
        .catch(error => {
            console.error('Error fetching training plan:', error);
        });
}

function updateTrainingPlanUI(planData) {
    const trainingPlanDetails = document.getElementById('training-plan-details');
    trainingPlanDetails.innerHTML = '';

    planData.forEach(item => {
        const domainName = item[0]; // Domain name is the first element
        const domainDetails = item[1]; // Domain details object is the second element

        const tr = document.createElement('tr');

        const tdDomain = document.createElement('td');
        tdDomain.textContent = domainName.replace(/_/g, ' '); // Replace underscores with spaces for readability

        const tdPerformance = document.createElement('td');
        tdPerformance.textContent = `${domainDetails.percentage.toFixed(2)}%`; // Extract 'percentage'

        // Determine recommendation based on performance percentage
        const recommendation = determineRecommendation(domainDetails.percentage);

        const tdRecommendation = document.createElement('td');
        tdRecommendation.textContent = recommendation;

        tr.appendChild(tdDomain);
        tr.appendChild(tdPerformance);
        tr.appendChild(tdRecommendation);

        trainingPlanDetails.appendChild(tr);
    });
}

function determineRecommendation(percentage) {
    // Example recommendation logic based on performance
    if (percentage >= 80) {
        return "Maintain current level";
    } else if (percentage >= 60) {
        return "Review key areas";
    } else {
        return "Intensive study recommended";
    }
}

function processDataForBarChart(planData) {
    let chartLabels = []; // Array to hold the labels for the chart (domain names)
    let performanceData = []; // Array to hold the performance data
    let backgroundColors = []; // Array to hold the background colors for each bar

    const colors = [
        'rgba(255, 99, 132, 0.5)',   // Red
        'rgba(54, 162, 235, 0.5)',   // Blue
        'rgba(255, 206, 86, 0.5)',   // Yellow
        'rgba(75, 192, 192, 0.5)',   // Green
        'rgba(153, 102, 255, 0.5)',  // Purple
        'rgba(255, 159, 64, 0.5)',   // Orange
        'rgba(96, 69, 96, 0.5)',     // Pink
        'rgba(128, 0, 128, 0.5)'     // Purple
        // Add more colors if there are more domains than colors
    ];

    planData.forEach((item, index) => {
        const domainName = item[0].replace(/_/g, ' '); // Replace underscores with spaces for readability
        const domainPerformance = item[1].percentage; // Extract the performance percentage

        chartLabels.push(domainName);
        performanceData.push(domainPerformance);

        // Assign a color to each bar
        let colorIndex = index % colors.length;
        backgroundColors.push(colors[colorIndex]);
    });

    return {
        labels: chartLabels,
        datasets: [{
            label: 'Domain Performance (%)',
            data: performanceData,
            backgroundColor: backgroundColors
        }]
    };
}

function createBarChart(barChartData) {
    const ctx = document.getElementById('myChart').getContext('2d');

    if (window.myBarChart) {
        window.myBarChart.destroy();
    }

    window.myBarChart = new Chart(ctx, {
        type: 'bar',
        data: barChartData,
        options: {
            scales: {
                y: {
                    beginAtZero: true,
                    max: 150, // Set the maximum value of the y-axis to 150%
                    ticks: {
                        callback: function(value) {
                            return value + "%";
                        }
                    }
                }
            },
            plugins: {
                legend: {
                    display: true
                },
                tooltip: {
                    enabled: true
                }
            },
            responsive: true,
            maintainAspectRatio: false,
        }
    });
}

function updateTestBreakdownUI(testBreakdownData) {
    const testBreakdownDetails = document.getElementById('test-breakdown-details');
    testBreakdownDetails.innerHTML = '';

    testBreakdownData.forEach(item => {
        const tr = document.createElement('tr');

        const tdDomain = document.createElement('td');
        tdDomain.textContent = item.domain; // Adjust according to your data structure

        const tdTestFocus = document.createElement('td');
        tdTestFocus.textContent = item.testFocus; // Adjust according to your data structure

        tr.appendChild(tdDomain);
        tr.appendChild(tdTestFocus);

        testBreakdownDetails.appendChild(tr);
    });
}


